import 'dart:async';
import 'dart:math';
import 'package:flutter/material.dart';
import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:factory_signage_app/const/colors.dart';
import 'package:factory_signage_app/const/config.dart';
import 'package:factory_signage_app/const/dimens.dart';
import 'package:factory_signage_app/const/string.dart';
import 'package:factory_signage_app/provider.dart';

enum LineStatus { confirmation, inactive, active }

extension on LineStatus {
  String get displayName {
    switch (this) {
      case LineStatus.confirmation:
        return lineStatuses[0];
      case LineStatus.inactive:
        return lineStatuses[1];
      case LineStatus.active:
        return lineStatuses[2];
    }
  }

  Color get color {
    switch (this) {
      case LineStatus.confirmation:
        return errorColor;
      case LineStatus.inactive:
        return warningColor;
      case LineStatus.active:
        return runningColor;
    }
  }

  Color get backgroundColor {
    switch (this) {
      case LineStatus.confirmation:
        return errorColor;
      case LineStatus.inactive:
        return normalColor;
      case LineStatus.active:
        return normalColor;
    }
  }
}

class Lines extends ConsumerStatefulWidget {
  const Lines(
      {super.key, this.size = const Size(double.infinity, double.infinity)});

  final Size size;

  @override
  LinesState createState() => LinesState();
}

class LinesState extends ConsumerState<Lines> {
  late Timer _timer;

  @override
  void initState() {
    super.initState();

    _timer = Timer.periodic(
      const Duration(seconds: intervalUpdateSignal),
      (timer) => changeSignal(),
    );
  }

  @override
  void dispose() {
    _timer.cancel();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final Size widgetSize = MediaQuery.of(context).size;

    return SizedBox(
      width: widget.size.width,
      height: widget.size.height,
      child: Row(
        mainAxisAlignment: MainAxisAlignment.spaceAround,
        children: [...lineWidgets(widgetSize)],
      ),
    );
  }

  void changeSignal() {
    final lineSignalStatus = createLineSignalStatus();
    ref.watch(lineSignalStatusProvider.notifier).state = lineSignalStatus;
  }

  List<Widget> lineWidgets(Size widgetSize) {
    final List<Widget> lines = [];
    for (int i = 0; i < lineNames.length; i++) {
      lines.add(lineWidget(widgetSize, i));
    }
    return lines;
  }

  Widget lineWidget(Size widgetSize, int num) {
    return Container(
      width: lineWidthRatio * widgetSize.width,
      height: linesHeightRatio * widgetSize.height,
      color: lineBackgroundColor,
      child: Stack(
        children: [
          Positioned(
            left: lineSignalPaddingWidthRatio * widgetSize.width,
            top: lineSignalPaddingHeightRatio * widgetSize.height,
            child: lineSignalWidget(widgetSize, num),
          ),
          Positioned(
            left: lineNamePaddingWidthRatio * widgetSize.width,
            top: lineNamePaddingHeightRatio * widgetSize.height,
            child: lineNameWidget(widgetSize, num),
          ),
          Positioned(
            left: lineStatusPaddingWidthRatio * widgetSize.width,
            top: lineStatusPaddingHeightRatio * widgetSize.height,
            child: lineStatusWidget(widgetSize, num),
          ),
        ],
      ),
    );
  }

  Widget lineSignalWidget(Size widgetSize, int num) {
    return Column(
      children: [
        ...signalWidgets(widgetSize, num),
      ],
    );
  }

  Widget lineNameWidget(Size widgetSize, int num) {
    return Container(
      alignment: Alignment.center,
      width: lineNameWidthRatio * widgetSize.width,
      child: Text(
        lineNames[num],
        style: TextStyle(
          color: textWhiteColor,
          fontSize: lineNameFontSizeRatio * widgetSize.height,
        ),
      ),
    );
  }

  Widget lineStatusWidget(Size widgetSize, int num) {
    final lineSignalStatus = ref.watch(lineSignalStatusProvider);
    return Container(
      alignment: Alignment.center,
      color: lineSignalStatus[num].backgroundColor,
      width: lineStatusWidthRatio * widgetSize.width,
      height: lineStatusHeightRatio * widgetSize.height,
      child: Text(
        lineSignalStatus[num].displayName,
        style: TextStyle(
          color: textWhiteColor,
          fontSize: lineStatusFontSizeRatio * widgetSize.height,
        ),
      ),
    );
  }

  List<Widget> signalWidgets(Size widgetSize, int num) {
    List<Widget> signals = [];
    final lineSignalStatus = ref.watch(lineSignalStatusProvider);
    for (final lineState in LineStatus.values) {
      signals.add(
        signalWidget(
          widgetSize,
          lineState.color,
          lineSignalStatus[num] == lineState
              ? activeSignalOpacity
              : inactiveSignalOpacity,
        ),
      );
    }
    return signals;
  }

  Widget signalWidget(Size widgetSize, Color color, double opacity) {
    return Container(
      width: widgetSize.width * lineSignalWidthRatio,
      height: widgetSize.height * lineSignalHeightRatio,
      color: color.withOpacity(opacity),
    );
  }
}

List<LineStatus> createLineSignalStatus() {
  final lineSignal = List.generate(
    lineNames.length,
    (i) => LineStatus.active,
    growable: false,
  );
  // One line for confirmation required and one line for inactive, otherwise in active.
  while (true) {
    final confirmationLine = Random().nextInt(lineNames.length);
    final inactiveLine = Random().nextInt(lineNames.length);
    if (confirmationLine != inactiveLine) {
      lineSignal[confirmationLine] = LineStatus.confirmation;
      lineSignal[inactiveLine] = LineStatus.inactive;
      break;
    }
  }
  return lineSignal;
}
